package com.jz.agent.client.service;

import com.google.gson.Gson;
import com.jz.agent.client.DmpDsAgentClient;
import com.jz.agent.config.DmpDsAgentConstants;
import com.jz.agent.utils.JDBCUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import org.apache.hadoop.security.UserGroupInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.PrivilegedAction;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@SuppressWarnings("unused")
public class DmpImpalaAgentClientService implements DmpDsAgentClient {
	
	private static String JDBC_DRIVER = "com.cloudera.impala.jdbc41.Driver";
	private Gson gson = new Gson();
	private static Logger logger = LoggerFactory.getLogger(DmpImpalaAgentClientService.class);
	
	static {
        try {
            Class.forName(JDBC_DRIVER);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

	protected String CONNECTION_URL;

	public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {
		/*boolean flag = false;
		Connection connection = null;
		try {
			connection = getConnection(ds);
			if(connection != null) flag = true;
			
		} catch (Exception e) {
            e.printStackTrace();
        } finally {
        	if(connection != null)
        		JDBCUtils.disconnect(connection, null, null);
        }
		return flag;*/
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
	}
	
	private Connection getConnection(DmpAgentDatasourceInfo ds) {
        Connection connection = null;
        ResultSet rs = null;
        PreparedStatement ps = null;
        CONNECTION_URL = ds.getJdbcUrl();
        try {
        	if(DmpDsAgentConstants.IS_ENABLED_KERBEROS) {
        		UserGroupInformation loginUser = JDBCUtils.initKerberosENV4IMPALA();
        		CONNECTION_URL = CONNECTION_URL + ";AuthMech=1;KrbRealm=" 
        				+ DmpDsAgentConstants.KERBEROS_KEYTAB_ACCOUNT_IMPALA 
        				+ ";KrbHostFQDN=" + ds.getAccessId() + ";KrbServiceName=impala";
        		connection = (Connection) loginUser.doAs(new PrivilegedAction<Object>(){
                    public Object run() {
                        Connection connection = null;
                        ResultSet rs = null;
                        PreparedStatement ps = null;
                        try {
                            Class.forName(JDBC_DRIVER);
                            connection = DriverManager.getConnection(CONNECTION_URL);
                        } catch (Exception e) {
                            e.printStackTrace();
                        } finally {
                            JDBCUtils.disconnect(connection, rs, ps);
                        }
                        return connection;
                    }
                });
        	} else {
        		connection = DriverManager.getConnection(CONNECTION_URL);
        	}
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
        	if(rs != null)
				try {
					rs.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(ps != null)
				try {
					ps.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(connection != null)
				try {
					connection.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        }
        return connection;
	}

	public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
		String sql = "show tables"; // like '*likename*'
		Connection conn = null;
		ResultSet rs = null;
        PreparedStatement ps = null;
		List<String> list = new ArrayList<String>();
		try {
			conn = getConnection(ds);
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				list.add(rs.getString(1).toLowerCase());
			}
		} catch (Exception e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "Impala获取表列表异常："+e.getMessage());
        } finally {
        	if(rs != null)
				try {
					rs.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(ps != null)
				try {
					ps.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(conn != null)
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        }
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
	}

	public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {
		String sql = "desc " + tableName;
		Connection conn = null;
		ResultSet rs = null;
        PreparedStatement ps = null;
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		try {
			conn = getConnection(ds);
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				Map<String, Object> row = new HashMap<String, Object>();
				row.put("name", rs.getString(1).toLowerCase());
				row.put("type", rs.getString(2).toUpperCase());
				row.put("comment", rs.getString(3));
				list.add(row);
			}
		} catch (Exception e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "Impala获取表字段信息异常："+e.getMessage());
        } finally {
        	if(rs != null)
				try {
					rs.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(ps != null)
				try {
					ps.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(conn != null)
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        }
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
	}

	@SuppressWarnings("resource")
	public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
		Map<String, Object> map = new HashMap<String, Object>();
		String sql = "select * from " + tableName + " limit 5";
		String sql1 = "desc " + tableName;
		
		Connection conn = null;
		ResultSet rs = null;
        PreparedStatement ps = null;
		try {
			conn = getConnection(ds);
			ps = conn.prepareStatement(sql1);
			rs = ps.executeQuery();
			List<String> headerList = new ArrayList<String>();
			while (rs.next()) {
				headerList.add(rs.getString(1).toLowerCase());
			}
			map.put("header", headerList);
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			List<List<Object>> resultList = new ArrayList<List<Object>>();
			while (rs.next()) {
				List<Object> row = new ArrayList<Object>();
				for (int i = 1; i <= headerList.size(); i++) {
					row.add(rs.getObject(i));
				}
				resultList.add(row);
			}
			map.put("result", resultList);
		} catch (Exception e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "IMPALA数据预览异常："+e.getMessage());
        } finally {
        	if(rs != null)
				try {
					rs.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(ps != null)
				try {
					ps.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        	if(conn != null)
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
        }
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
	}
	
}
